/*
 * Copyright (C) 2013, 2014 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Nekhelesh Ramananthan <krnekhelesh@gmail.com>
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.Popups 0.1
import Ubuntu.Components.Themes.Ambiance 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import "../common/ClockUtils.js" as Utils
import "../common"

/*
  Page to edit or create a new alarm
  Parameters passed:

  Variable to check if the page is for creating a new alarm or editing an existing one
  bool isNewAlarm

  Variable to hold the index of the alarm which is being edited. If page is called for creating a new alarm, this variable will not be available
  int alarmIndex
 */
Page {
    id: addAlarmPage
    objectName: "addAlarmPage"

    // Property to determine if this is New/Edit mode
    property bool isNewAlarm

    // Property to store the index of the alarm to be edited
    property int alarmIndex

    // Property to store the focus property of the alarm name field
    property bool setFocus: false

    // Property to store the AM/PM mode while in the 12-hour time format
    property bool isPm

    // Property to retrieve and store the time format setting of the user
    property string timeFormat: appSetting.contents.timeFormat
    property bool isMilitaryFormat: timeFormat === "24-hour"

    visible: false
    flickable: null

    Component.onCompleted: {
        // Set focus to alarm name label when navigating using keyboard
        if(setFocus)
            nameAlarm.forceActiveFocus()

        if (isNewAlarm){
            addAlarmPage.title = i18n.tr("Add Alarm")
            showDefaultTime()
            addAlarmFace.hours = Number(hourLabel.text) // Set hour hand
        }
        else {
            addAlarmPage.title = i18n.tr("Edit Alarm")
            readAlarm()
        }
    }

    Alarm {
        id: alarm
        onStatusChanged: {
            if (status != Alarm.Ready)
                return;
            if ((operation > Alarm.NoOperation) && (operation < Alarm.Reseting)) {
                pagestack.pop();
            }
        }
    }

    // Function to validate if the alarm was saved properly
    function validateAlarm(alarmObject) {
        if (alarmObject.error !== Alarm.NoError) {
            Utils.log("Error saving alarm, code: " + alarmObject.error)
            if (alarmObject.error === 2)
                errorLabel.text = i18n.tr("Please select a time in the future")
            else
                errorLabel.text = i18n.tr("Unable to set alarm")
            errorLabel.visible = true
            return false
        }
        else
            return true
    }

    // Function to show default time when creating new alarm ("00:00" or "12:00")
    function showDefaultTime() {
        hourLabel.text = isMilitaryFormat ? "00" : "12"
        minuteLabel.text = "00"
    }

    // Function to set the correct alarm time depending on AM/PM setting
    function setAlarmTime(alarmTime) {
        var hour = Number(hourLabel.text)
        var minute = Number(minuteLabel.text)

        if(isPm && !isMilitaryFormat && hourLabel.text != "12"){
            alarmTime.setHours(hour + 12, minute, 0)
        }
        else if(!isPm && !isMilitaryFormat && hourLabel.text == "12"){
            alarmTime.setHours(0, minute, 0)
        }
        else {
            alarmTime.setHours(hour, minute, 0)
        }
    }

    // Function to save a new alarm
    function saveNewAlarm() {
        var alarmTime = new Date();

        setAlarmTime(alarmTime)
        alarm.message = nameAlarm.text
        alarm.date = alarmTime
        alarm.enabled = true
        alarm.save()
        validateAlarm(alarm)
    }

    // Function to read and show an existing alarm
    function readAlarm() {
        var temp_alarm = alarmModel.get(alarmIndex)

        alarm.message = temp_alarm.message
        alarm.type = temp_alarm.type
        alarm.daysOfWeek = temp_alarm.daysOfWeek
        alarm.enabled = temp_alarm.enabled
        alarm.date = temp_alarm.date

        // Determining if the alarm read is set to trigger in the morning or evening
        isPm = Qt.formatTime(alarm.date, "AP") === "PM"
        hourLabel.text = isMilitaryFormat ? Qt.formatTime(alarm.date, "hh") : Qt.formatTime(alarm.date, "h AP").split(" ")[0]
        minuteLabel.text = Qt.formatTime(alarm.date, "mm")
        addAlarmFace.hours = Number(hourLabel.text) // Set hour hand
    }

    // Function to update an existing alarm
    function updateAlarm() {
        var temp_alarm = alarmModel.get(alarmIndex)

        var alarmTime = new Date()
        setAlarmTime(alarmTime)

        temp_alarm.message = nameAlarm.text
        temp_alarm.type = alarm.type
        temp_alarm.daysOfWeek = alarm.daysOfWeek
        temp_alarm.date = alarmTime
        temp_alarm.save()

        if(validateAlarm(temp_alarm))
            pagestack.pop()
    }

    // Function to print hour label
    function setHoursLabel(hours, maxValue) {
        var z
        if (maxValue === 24){
            z = 2
            if (addAlarmFace.hours > (maxValue - 1)) {
                hours = 0
            }
        }
        else {
            z = 1
            if (addAlarmFace.hours < 1){
                hours = 12
            }
        }
        addAlarmFace.hours = Math.round(hours)

        return Utils.zeroleft(hours, z);
    }

    // Function to print minute label
    function setMinutesLabel(minutes) {
        if (addAlarmFace.minutes > 59) {
            minutes = 0
        }
        addAlarmFace.minutes = Math.round(minutes)

        return Utils.zeroleft(minutes, 2);
    }

    // Function to update hour on minute hand rotation
    function updateHoursOnMinuteRotation(){
        var hour = Number(hourLabel.text)
        var minute = Number(minuteLabel.text)
        var maxHourValue = isMilitaryFormat ? 23 : 12
        var baseHourValue = isMilitaryFormat ? 0 : 12

        if (minute === 0 && addAlarmFace.valueWas >= 58){
            if(hour === baseHourValue - 1 && !isMilitaryFormat){
                isPm = !isPm
            }
            if (hour > maxHourValue - 1){
                hourLabel.text = isMilitaryFormat ? "00" : "12"

                if (hour === baseHourValue && !isMilitaryFormat){
                    hourLabel.text = "1"
                }
            }
            else {
                hourLabel.text = isMilitaryFormat ? ("0" + (hour + 1)).slice(-2) :  hour + 1
            }
        }
        else if (minute >= 58 && addAlarmFace.valueWas === 0){
            if (hour === baseHourValue && !isMilitaryFormat){
                isPm = !isPm
            }
            if (hour <= 1){
                hourLabel.text = isMilitaryFormat ? "00" : "12"
                if (hour === baseHourValue && isMilitaryFormat){
                    hourLabel.text = "23"
                }
            }
            else {
                hourLabel.text = isMilitaryFormat ? ("0" + (hour - 1)).slice(-2) : hour - 1
            }
        }
        addAlarmFace.valueWas = minute
    }

    // Function to update AM/PM indicator on the hour hands rotation
    function updateAmPmOnRotation(){
        var hour = Number(hourLabel.text)
        if (((hour === 11 && addAlarmFace.valueWas === 12) || (hour === 12 && addAlarmFace.valueWas === 11)) && !isMilitaryFormat ){
            isPm = !isPm
        }
        addAlarmFace.valueWas = hour
    }

    // Timer to show the error message for one second and then automatically hiding it
    Timer {
        id: errorTimer
        interval: 1000
        running: true
        onTriggered: errorLabel.visible = false
    }

    Flickable {
        clip: true
        anchors.fill: parent
        contentHeight: mainColumn.height + units.gu(5)
        interactive: contentHeight > parent.height

        Column {
            id: mainColumn

            spacing: units.gu(1)
            anchors { left: parent.left; right: parent.right; top: parent.top; topMargin: units.gu(5)}

            ClockBase {
                id: addAlarmFace

                draggable: true
                enableMouseArea: false
                showSecondHand: false
                showMinuteHand: false
                innerLabel.visible: false
                anchors.horizontalCenter: parent.horizontalCenter
                maximumValue: isMilitaryFormat ? 24 : 12

                onMinutesChanged: minuteLabel.text = setMinutesLabel(minutes)
                onHoursChanged: {hourLabel.text = setHoursLabel(hours, maximumValue); updateAmPmOnRotation() }

                states: [
                    State {
                        name: "minuteMode"
                        PropertyChanges { target: addAlarmFace; maximumValue: 60; showHourHand: false; showMinuteHand: true }
                        PropertyChanges { target: hourLabel; font.pixelSize: units.dp(41); color: "White" }
                        PropertyChanges { target: minuteLabel; font.pixelSize: units.dp(55); color: "LightGreen" }
                    }
                ]

                Image {
                    id: alarmHintImage
                    smooth: true
                    width: units.gu(40)
                    anchors.centerIn: parent
                    fillMode: Image.PreserveAspectFit
                    source: Qt.resolvedUrl("../images/alarm_black.png")
                }

                centerContent: [
                    Column {
                        id: alarmLabelColumn

                        anchors.centerIn: parent
                        visible: !errorLabel.visible

                        // TODO: Investigate if it is worth moving this as a separate component when implementing the Digital Mode
                        Row {
                            id: alarmTimeRow

                            Label {
                                id: hourLabel
                                color: "LightGreen"
                                font.pixelSize: units.dp(55)
                                anchors.verticalCenter: parent.verticalCenter

                                MouseArea {
                                    anchors.fill: parent
                                    preventStealing: true
                                    onClicked: {
                                        addAlarmFace.state = "";
                                        addAlarmFace.hours = Number(hourLabel.text)
                                    }
                                }

                                Behavior on font.pixelSize {
                                    UbuntuNumberAnimation { duration: UbuntuAnimation.BriskDuration }
                                }
                            }

                            Label {
                                text: ":"
                                font.pixelSize: units.dp(41)
                                anchors.verticalCenter: parent.verticalCenter
                            }

                            Label {
                                id: minuteLabel
                                font.pixelSize: units.dp(41)
                                anchors.verticalCenter: parent.verticalCenter
                                onTextChanged: if(addAlarmFace.state == "minuteMode") updateHoursOnMinuteRotation()

                                MouseArea {
                                    id: minuteMouseArea
                                    anchors.fill: parent
                                    onClicked: {
                                        addAlarmFace.state = "minuteMode";
                                        addAlarmFace.minutes = Number(minuteLabel.text)
                                    }
                                }

                                Behavior on font.pixelSize {
                                    UbuntuNumberAnimation { duration: UbuntuAnimation.BriskDuration }
                                }
                            }
                        }

                        UbuntuShape {
                            radius: "medium"
                            width: childrenRect.width
                            height: childrenRect.height
                            visible: !isMilitaryFormat
                            anchors.horizontalCenter: parent.horizontalCenter
                            Label {
                                id: timeFormatLabel
                                font.pixelSize: units.dp(31)
                                text: isPm ? "PM" : "AM"
                            }
                            MouseArea {
                                anchors.fill: parent
                                onClicked: isPm = !isPm
                            }
                        }
                    }
                ]

                Repeater {
                    id:hourMarkers

                    model: !isMilitaryFormat ? 12 : 24

                    delegate: AnalogClockMarker {
                        size: 0.2
                        distance: 0.45
                        number: modelData
                        color: "transparent"
                        total: hourMarkers.model
                        visible: !errorLabel.visible && addAlarmFace.state === ""
                        Label {
                            opacity: 0.5
                            fontSize: "small"
                            anchors.centerIn: parent
                            text: modelData === 0 && !isMilitaryFormat ? 12 : modelData
                        }
                    }
                }

                Repeater {
                    id:minuteMarkers

                    model: 60

                    delegate: AnalogClockMarker {
                        size: 0.2
                        distance: 0.45
                        number: modelData
                        color: "transparent"
                        total: minuteMarkers.model
                        visible: !errorLabel.visible && addAlarmFace.state === "minuteMode"
                        Label {
                            opacity: 0.5
                            fontSize: "x-small"
                            text: modelData
                            anchors.centerIn: parent
                            visible: modelData % 5 === 0
                        }
                    }
                }

                Label {
                    id: errorLabel
                    visible: false
                    onVisibleChanged: visible === true ? errorTimer.start() : undefined
                    parent: addAlarmFace.centerItem
                    anchors {
                        centerIn: parent
                        fill: parent
                        margins: units.gu(2)
                    }
                    horizontalAlignment: Text.AlignHCenter
                    verticalAlignment: Text.AlignVCenter
                    color: Theme.palette.normal.baseText
                    fontSize: "large"
                    text: i18n.tr("Unable to set alarm")
                    wrapMode: Text.WordWrap
                }
            }

            // Element to set the name of alarm
            LabelDots {
                id: nameAlarm
                objectName: "addAlarmName"

                text: alarm.message
                width: length < (addAlarmFace.width - units.gu(4))/units.gu(1.5)  ? length * units.gu(1.5) + units.gu(4) : addAlarmFace.width;
                placeholderText: i18n.tr("Alarm Name")
                anchors.horizontalCenter: addAlarmFace.horizontalCenter
                onClickedOutside: mainColumn.forceActiveFocus()
            }

            ListItem.ThinDivider { }

            Column {
                id: alarmOptions

                anchors { left: parent.left; leftMargin: units.gu(2); right: parent.right; rightMargin: units.gu(2) }
                spacing: units.gu(1)

                ListItem.Standard {
                    id: repeatSelector
                    anchors { left: parent.left; right: parent.right; margins: -units.gu(2) }
                    text: i18n.tr("Repeats")

                    control: Switch {
                        id: repeatSwitch
                        objectName: "RepeatSelector"
                        checked:  {
                            if (alarm.type == Alarm.OneTime)
                                return false;
                            else if (alarm.type == Alarm.Repeating) {
                                return true;
                            }
                        }
                        onCheckedChanged: {
                            switch (checked) {
                            case false:
                                alarm.type = Alarm.OneTime;
                                break;
                            case true:
                                alarm.type = Alarm.Repeating;
                                break;
                            }
                        }
                    }
                }

                OptionSelector {
                    id: occurSelector
                    objectName: "OccursSelector"
                    text: i18n.tr("Occurs")
                    visible: repeatSwitch.checked == false  // Show the day selector only when the alarm is a one-type alarm
                    containerHeight: itemHeight * 4
                    model: [Qt.locale().standaloneDayName(0), Qt.locale().standaloneDayName(1), Qt.locale().standaloneDayName(2), Qt.locale().standaloneDayName(3), Qt.locale().standaloneDayName(4), Qt.locale().standaloneDayName(5), Qt.locale().standaloneDayName(6)]

                    selectedIndex: alarm.date.getDay();

                    onSelectedIndexChanged: {
                        switch (selectedIndex) {
                        case 0:
                            alarm.daysOfWeek = Alarm.Sunday;
                            break;
                        case 1:
                            alarm.daysOfWeek = Alarm.Monday;
                            break;
                        case 2:
                            alarm.daysOfWeek = Alarm.Tuesday;
                            break;
                        case 3:
                            alarm.daysOfWeek = Alarm.Wednesday;
                            break;
                        case 4:
                            alarm.daysOfWeek = Alarm.Thursday;
                            break;
                        case 5:
                            alarm.daysOfWeek = Alarm.Friday;
                            break;
                        case 6:
                            alarm.daysOfWeek = Alarm.Saturday;
                            break;
                        }
                    }
                }

                ListItem.MultiValue {
                    id: multipleOccurSelector
                    anchors { left: parent.left; right: parent.right; margins: -units.gu(2) }
                    text: i18n.tr("Occurrence")
                    objectName: "OccurrenceList"
                    values: getValues()
                    visible: repeatSwitch.checked == true
                    onClicked: PopupUtils.open(Qt.resolvedUrl("AlarmDays.qml"), multipleOccurSelector, {"alarm": alarm});

                    function getValues() {
                        var occurs = [];
                        if (alarm.daysOfWeek & Alarm.Monday) occurs.push(Qt.locale().standaloneDayName(1));
                        if (alarm.daysOfWeek & Alarm.Tuesday) occurs.push(Qt.locale().standaloneDayName(2));
                        if (alarm.daysOfWeek & Alarm.Wednesday) occurs.push(Qt.locale().standaloneDayName(3));
                        if (alarm.daysOfWeek & Alarm.Thursday) occurs.push(Qt.locale().standaloneDayName(4));
                        if (alarm.daysOfWeek & Alarm.Friday) occurs.push(Qt.locale().standaloneDayName(5));
                        if (alarm.daysOfWeek & Alarm.Saturday) occurs.push(Qt.locale().standaloneDayName(6));
                        if (alarm.daysOfWeek & Alarm.Sunday) occurs.push(Qt.locale().standaloneDayName(7));
                        return occurs;
                    }
                }
            }
        }
    }

    // Toolbar to show back and save alarm button
    tools: ToolbarItems {
        id: toolbarAddAlarm

        opened: true
        locked: true

        ToolbarButton {
            id: addPresetToolbarButton
            objectName: "saveAlarmButton"
            iconSource: Qt.resolvedUrl("../images/select_icon.png")
            text: i18n.tr("Save")
            onTriggered: {
                if (addAlarmPage.isNewAlarm)
                    saveNewAlarm()
                else
                    updateAlarm()
            }
        }

    }
}

