/*
 * Copyright (C) 2013, 2014 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Juha Ristolainen <juha.ristolainen@codemancers.fi>
 *              Nekhelesh Ramananthan <krnekhelesh@gmail.com>
 *              Nick Leppänen Larsson <frals@frals.se>
 */

import QtQuick 2.0
import U1db 1.0 as U1db
import Ubuntu.Components 0.1
import QtQuick.XmlListModel 2.0
import Ubuntu.Components.ListItems 0.1 as ListItem
import "../common/ClockUtils.js" as Utils
import "../common"

Page {
    id: clockPage

    // Property to hold the formatted time string to show on the screen
    property string currentTimeFormatted
    property string currentDate
    property real currentUTCTime
    property real diff: new Date().getTimezoneOffset()

    // Property to store the user location (city, longitude, latitude)
    property var userLocation: { "city": i18n.tr("Retrieving current location..."), "lng": 200, "lat": 200 }

    // U1dB document to store the current location of the user
    U1db.Document {
        id: currentLocation
        database: db
        create: true
        docId: "currentUserLocation"        
        defaults: { "city": "undefined", "lat": 200, "lng": 200 }        
    }

    // U1db Index to index all documents storing the world location details
    U1db.Index {
        database: db
        id: by_worldclock
        expression: ["worldlocation.city", "worldlocation.rawOffSet", "worldlocation.longitude", "worldlocation.latitude"]
    }

    U1db.Query {
        id: worldClock
        index: by_worldclock
        query: ["*","*", "*", "*"]
    }

    Component.onCompleted: {
        Utils.log("ClockPage loaded");
        currentTimeFormatted = Utils.convertTime(new Date().getHours(), new Date().getMinutes(), new Date().getUTCSeconds(), appSetting.contents.timeFormat)
        updateTime();

        if (currentLocation.contents.city !== "undefined") {
            userLocation = {
                city: currentLocation.contents.city,
                lng: currentLocation.contents.lng,
                lat: currentLocation.contents.lat
            }
        }

        Utils.log("Saved Current Location: " + currentLocation.contents.city);
        Utils.log("Saved Current Location Coordinates: " + currentLocation.contents.lng + "," + currentLocation.contents.lat)
    }

    actions: [
        Action {
            id: addCityAction
            text: i18n.tr("Add City")
            // TRANSLATORS: this is a list of semicolon-separated keywords to be used to activate this action (Add City) in the HUD
            keywords: i18n.tr("Add;Timezone;Timezones;World;City;Cities;Town;Towns;Place;Places;Location;Locations;Time;Locale;Local;Current")
            description: "Add a world city"
            iconSource: Qt.resolvedUrl("../images/add_icon.png")
            onTriggered: pagestack.push(Qt.resolvedUrl("WorldClock.qml"), {"isWorldCity": true})
        }
    ]

    // Function to save a world location to u1db database
    function addWorldLocation(cityName, diff, longitude, latitude) {
        Utils.log("Adding world location")
        db.putDoc({ "worldlocation": { "city": cityName, "rawOffSet": diff, "longitude": longitude, "latitude": latitude }}, encodeURIComponent(cityName+"_"+longitude+"_"+latitude))
    }

    // Function to update current location of the userand save it to the database
    function updateCurrentLocation(cityname, longitude, latitude) {
        currentLocation.contents = {
            city: cityname,
            lng: longitude,
            lat: latitude
        }

        userLocation = {
            city: cityname,
            lng: longitude,
            lat: latitude
        }
    }

    // Function which runs every second to update the clock label
    function onTimerUpdate(now) {
        currentTimeFormatted = Utils.convertTime(now.getHours(), now.getMinutes(), now.getUTCSeconds(), appSetting.contents.timeFormat)
        clockFace.timeChanged(now)
        if (now.getUTCSeconds() == 0) {
            updateTime();
        }
    }

    // Function which runs every minute to update the world time
    function updateTime() {
        var now = new Date()
        currentDate = Qt.formatDate(now, "ddd d, MMM")
        now.setMinutes(now.getMinutes() + diff)
        currentUTCTime = now.getTime()
    }

    Connections {
        target: root
        onApplicationStateChanged: {
            // Update world time immediately when the clock app is brought from suspend
            // instead of waiting for the next minute to update.
            if(root.applicationState)
                updateTime()
        }
    }

    GeoIPModel {
        id: geoIP
        onStatusChanged: {
            if (status == XmlListModel.Ready && geoIP.get(0).city !== currentLocation.contents.city && geoIP.get(0).city !== "None") {
                Utils.log("Retrieved current location using GeoIP. Writing into database and setting it as current location")
                updateCurrentLocation(geoIP.get(0).city, geoIP.get(0).lng, geoIP.get(0).lat);
            }
            else if (status == XmlListModel.Ready && geoIP.get(0).city === "None") {
                if (currentLocation.contents.city === "undefined") {
                    userLocation.city = i18n.tr("Set current location name")
                    currentLocation.progression = true
                }
            }
            else if(status == XmlListModel.Error) {
                Utils.error("Unable to retrieve GeoIP Data -> " + geoIP.errorString())
                if (currentLocation.contents.city === "undefined") {
                    userLocation.city = i18n.tr("Set current location name")
                }
            }
        }
    }

    Flickable {
        id: clockAnimationContainer
        objectName: "animationContainer"

        clip: true;
        anchors.fill: parent
        contentWidth: parent.width
        contentHeight: clockFace.height + clockFace.anchors.topMargin + savedWorldClock.height + savedWorldClock.anchors.topMargin + units.gu(3)

        // QML Element to draw the analogue clock face along with its hour, minute and second hands.
        ClockBase {
            id: clockFace

            anchors {
                top: parent.top
                topMargin: units.gu(10)
                horizontalCenter: parent.horizontalCenter
            }

            innerLabel.text: currentTimeFormatted
            innerLabel.font.pixelSize: appSetting.contents.timeFormat === "12-hour" ? units.dp(31): units.dp(41)

            hours: new Date().getHours() * 5
            minutes: new Date().getMinutes()
            seconds: new Date().getUTCSeconds()

            function timeChanged(now) {
                hours = now.getHours() * 5
                minutes = now.getMinutes()
                seconds = now.getUTCSeconds();
            }

            onClicked: {
                if (easterEggCircle.isReady == XmlListModel.Ready && currentLocation.contents.city !== "undefined")
                    clockFace.state == "" ? clockFace.state = "easteregg" : clockFace.state = "";
                else
                    Utils.error("Sunrise/Sunset times cannot be loaded without setting the current location or without a internet connection.")
            }

            EasterEgg {
                id: easterEggCircle
                parent: clockFace.centerItem
                anchors.centerIn: parent
                width: clockFace.centerItem.width; height: width;
                radius: width / 2;
                latitude: userLocation.lat
                longitude: userLocation.lng
            }

            states: [
                State {
                    name: "easteregg"
                    PropertyChanges { target: easterEggCircle; opacity: 1; }
                    PropertyChanges { target: clockFace.innerLabel; opacity: 0; }
                    PropertyChanges { target: easterEggCircle; sunriseLabel: easterEggCircle.getSunTime(easterEggCircle.model.get(0).sunriseTime); }
                    PropertyChanges { target: easterEggCircle; sunsetLabel: easterEggCircle.getSunTime(easterEggCircle.model.get(0).sunsetTime); }
                },
                State {
                    name: ""
                    PropertyChanges { target: clockFace.innerLabel; opacity: 1; }
                    PropertyChanges { target: easterEggCircle; opacity: 0; }
                }
            ]
        }

        Label {
            id: currentUserLocation

            text: userLocation.city
            fontSize: "large"
            font.bold: true
            anchors {
                horizontalCenter: clockFace.horizontalCenter
                top: clockFace.bottom
                topMargin: units.gu(5)
            }

            MouseArea {
                anchors.fill: parent
                onClicked: pagestack.push(Qt.resolvedUrl("WorldClock.qml"), {"isWorldCity": false})
            }
        }

        Label {
            id: currentDay
            text: currentDate
            anchors {
                horizontalCenter: clockFace.horizontalCenter
                top: currentUserLocation.bottom
            }
        }

        Column {
            id: savedWorldClock

            property int dynamicTopSpacing: clockPage.height - clockPage.header.height - clockFace.height - worldLocationHeader.height - clockFace.anchors.topMargin - divider.height;

            height: childrenRect.height;
            anchors { left:parent.left; right:parent.right; top: clockFace.bottom; topMargin: savedWorldClock.dynamicTopSpacing }

            ListItem.ThinDivider {
                id: divider
                visible: listWorldClocks.count !== 0
            }

            ListItem.Header {
                id: worldLocationHeader
                visible: listWorldClocks.count !== 0
                Label {
                    text: i18n.tr("World")
                    anchors { verticalCenter: parent.verticalCenter; left: parent.left; leftMargin: units.gu(2) }
                    color: Theme.palette.normal.baseText
                    fontSize: "medium"
                }
            }

            ListView {
                id: listWorldClocks
                objectName: "listWorldClocks"
                
                clip: true
                anchors { left: parent.left; right: parent.right }
                height: 5 * units.gu(6) // height is defined to show 5 items by default
                model: worldClock
                currentIndex: -1

                delegate: ListItem.Standard {
                    objectName: "savedworldcity" + index
                    Label {
                        id: worldCityName
                        text: model.contents.city
                        objectName: "city_name"
                        elide: Text.ElideRight
                        anchors { verticalCenter: parent.verticalCenter; left: parent.left; right: worldCityTime.left; margins: units.gu(3) }
                        color: Theme.palette.normal.baseText
                        fontSize: "large"
                    }

                    Label {
                        id: worldCityTime
                        text: appSetting.contents.timeFormat === "12-hour" ? Qt.formatTime(new Date(currentUTCTime + (model.contents.rawOffSet * 60000)), "h:mm AP") : Qt.formatTime(new Date(currentUTCTime + (model.contents.rawOffSet * 60000)), "hh:mm")
                        anchors { verticalCenter: parent.verticalCenter; right: parent.right; rightMargin: units.gu(2) }
                        color: Theme.palette.normal.baseText
                        fontSize: "large"
                    }

                    selected: listWorldClocks.currentIndex == index
                    removable: true
                    confirmRemoval: true

                    onItemRemoved: {
                        // NOTE: This causes the document to be deleted twice resulting in an error.
                        // The bug has been reported at https://bugs.launchpad.net/ubuntu-ui-toolkit/+bug/1276118
                        Utils.log("Deleting world location: " + model.contents.cityname)
                        db.putDoc("", model.docId)
                    }
                }
            }
        }
    }

    tools: ToolbarItems {
        id: toolbarClock

        ToolbarButton {
            id: settings
            action: appSettingsAction
        }

        ToolbarButton {
            id: addCity
            action: addCityAction
            objectName: "addCityAction"
        }
    }
}
