/*
 * Copyright (C) 2013, 2014 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Nekhelesh Ramananthan <krnekhelesh@gmail.com>
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import QtQuick.XmlListModel 2.0
import Ubuntu.Components.ListItems 0.1 as ListItem
import "../common/ClockUtils.js" as Utils

// Page to to search a world city and add it to the world clocks
Page {
    id: worldPage
    objectName: "worldPage"

    property bool isWorldCity;

    // Property to store the focus property of the search field
    property alias setFocus: searchBox.setFocus

    visible: false;
    title: isWorldCity ? i18n.tr("Add City") : i18n.tr("Edit Current Location")    

    Column {
        id: worldClocks

        // Properties to store the user searched city details
        property string city: "null";
        property real lng: 0;
        property real lat: 0;        

        // Function to clear user search and return to original state
        function clearUserSearch () {
            searchCityModel.source = searchCityModel.localCityUrl();
            searchBox.search_string = "";
            worldList.currentIndex = -1;
        }

        anchors { fill: parent; topMargin: units.gu(2) }
        height: childrenRect.height;
        spacing: units.gu(2);

        // XML Model to list cities provided locally with clock-app or search cities online and retrieve latitude and longitude of searched city
        WorldCity {
            id: searchCityModel
            search_string: searchBox.search_string
        }

        // XML Model to retrieve timezone of searched city
        CityTimezone {
            id: cityDetailsModel
            onStatusChanged: {
                if(status == XmlListModel.Ready && worldClocks.city != "null") {
                    if (isWorldCity)
                        clockPage.addWorldLocation(worldClocks.city, getTimeDifference(cityDetailsModel.get(0).time), worldClocks.lng, worldClocks.lat)
                    else
                        clockPage.updateCurrentLocation(worldClocks.city, worldClocks.lng, worldClocks.lat)
                    worldClocks.clearUserSearch();
                    pageStack.pop()
                }
            }
        }

        // Search Bar to search for cities (online)
        // TODO: Add capability to also search cities provided locally with clock-app
        SearchBox {
            id: searchBox
            objectName: "worldSearchBox"

            height: units.gu(4)
            anchors { left: parent.left; right: parent.right }

            onSearchTriggered: {
                if (search_string != "")
                    searchCityModel.source = searchCityModel.searchCityUrl(search_string);
                else
                    searchCityModel.source = searchCityModel.localCityUrl()
            }
        }

        // List to display the search results
        ListView {
            id: worldList
            objectName: "worldList"

            anchors { left: parent.left; right: parent.right }
            height: units.gu(50)    // TODO: Make list view dynamic rather than the fixed 50 units gu.
            model: searchCityModel
            currentIndex: -1
            clip: true;

            Component {
                id: sectionHeading
                ListItem.Header {
                    Label {
                        text: section
                        anchors { verticalCenter: parent.verticalCenter; left: parent.left; leftMargin: units.gu(2) }
                        color: Theme.palette.normal.baseText
                        fontSize: "medium"
                    }
                }
            }

            Label {
                id: errorMessage
                width: parent.width
                fontSize: "large"
                wrapMode: Text.WordWrap
                horizontalAlignment: TextInput.AlignHCenter
                anchors.horizontalCenter: parent.horizontalCenter
                visible: worldList.count == 0 || searchCityModel.status == XmlListModel.Error || cityDetailsModel.status == XmlListModel.Error
                text: searchCityModel.status == XmlListModel.Error || cityDetailsModel.status == XmlListModel.Error ? i18n.tr("Unable to search online \nPlease check your internet connection") : i18n.tr("No results found")
            }

            section.property: !errorMessage.visible ? "city" : ""
            section.criteria: ViewSection.FirstCharacter
            section.delegate: sectionHeading;
            section.labelPositioning: ViewSection.InlineLabels

            delegate: ListItem.Base {
                objectName: "worldcity" + index
                visible: !errorMessage.visible
                height: adminName || adminName2 ? units.gu(8) : units.gu(6)
                Column {
                    anchors { top: parent.top; topMargin: units.gu(0.5); left: parent.left; leftMargin: units.gu(3) }
                    Label {
                        id: cityDelegate
                        objectName: "city_name"
                        text: city
                        color: Theme.palette.normal.baseText
                        fontSize: "large"
                    }
                    Label {
                        id: stateDelegate
                        width: worldClocks.width
                        elide: Text.ElideRight
                        text: {
                            if (adminName && adminName2)
                                return adminName2 + ", " + adminName
                            else if(adminName)
                                return adminName
                            else if(adminName2)
                                return adminName2
                            else
                                return ""
                        }
                        visible: text == "" ? false : true
                        color: Theme.palette.normal.baseText
                        fontSize: "small"
                    }
                    Label {
                        id: countryDelegate
                        text: country
                        color: Theme.palette.normal.baseText
                        fontSize: "small"
                    }
                }

                selected: worldList.currentIndex == index;

                onClicked: {
                    worldList.currentIndex = index;
                    worldClocks.lat = searchCityModel.get(index).lat;
                    worldClocks.lng = searchCityModel.get(index).lng;
                    worldClocks.city = searchCityModel.get(index).city;
                    cityDetailsModel.source = cityDetailsModel.getCityTimezoneUrl(worldClocks.lat, worldClocks.lng)
                }
            }

            Scrollbar {
                flickableItem: worldList;
                align: Qt.AlignTrailing;
            }
        }
    }
}
