# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2013 Canonical Ltd.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# Authored by: Nekhelesh Ramananthan <krnekhelesh@gmail.com>
#              Nicholas Skaggs <nicholas.skaggs@canonical.com>

from ubuntuuitoolkit import emulators as toolkit_emulators
from autopilot.matchers import Eventually
from testtools.matchers import Not, Is, Equals
from time import sleep

class MainView(toolkit_emulators.MainView):

    # Common Emulator Functions
    def _drag_page(self, page, test_case, direction):
        """Function to drag the page up/down"""
        page = self.wait_select_single(page)
        animating = lambda: page.select_single("QQuickFlickable").moving

        #make sure we're not already moving :-)
        test_case.assertThat(animating, Eventually(Equals(False)))

        x, y, w, h = page.globalRect
        tx = x + (w / 2)
        ty = y + (h / 2)

        if direction == "down":
            self.pointing_device.drag(tx, ty, tx, ty + h / 3)
        else:
            self.pointing_device.drag(tx, ty, tx, ty - h / 3)

        test_case.assertThat(animating, Eventually(Equals(False)))

    def drag_page_up(self, page, test_case):
        """Drag the given page up."""
        self._drag_page(page, test_case, "up")

    def drag_page_down(self, page, test_case):
        """Drag the given page down."""
        self._drag_page(page, test_case, "down")

    def longpress_object(self, obj):
        """Function to longpress an object"""
        self.pointing_device.move_to_object(obj)
        self.pointing_device.press()
        sleep(2)
        self.pointing_device.release()

    # CLOCK Page Emulator Functions
    def get_clock_page(self):
        """Return the page containing the main clock."""
        return self.select_single("ClockPage")

    def get_time_label(self):
        """Return the label with the timer countdown"""
        return self.select_single("ClockPage").wait_select_single("Label", objectName="label")

    # WORLD CLOCK Page Emulator Functions
    def get_world_cities_page(self):
        """Return the page containing the world cities."""
        return self.wait_select_single("WorldClock")

    def get_search_box(self):
        """Returns the search box used to search for world cities online."""
        return self.wait_select_single('SearchBox', objectName="worldSearchBox")

    def get_world_cities_list(self):
        """Return the component containing the list of available world cities."""
        return self.wait_select_single(objectName="worldList")

    def get_city_name(self, city):
        """Return the name of the city passed as argument"""
        return city.wait_select_single('Label', objectName='city_name').text

    def get_saved_cities_list(self):
        """Return the component containing list of user saved world city locations."""
        return self.wait_select_single(objectName="listWorldClocks")

    def grab_first_listed_city(self):
        """Return the first city from the available world cities list (both local and online)"""
        return self.wait_select_single("Base", objectName="worldcity0")

    def select_city(self, city):
        """Click the selected city passed as argument"""
        self.pointing_device.click_object(city)

    def grab_first_saved_city(self):
        """Return the first city from the user saved world cities list"""
        return self.wait_select_single("Standard", objectName="savedworldcity0")

    # TIMER Page Emulator Functions
    def get_timer_reset_button(self):
        """Returns the select for the reset button"""
        return self.select_single("ImageButton", objectName="resetButtonTimer")

    def get_timer_name_preset(self):
        """Returns the TextField where insert the name of the preset"""
        return self.wait_select_single("LabelDots", objectName="namePreset")

    def get_timer_minute_hand(self):
        """Returns the hour hand of clock in timer tab"""
        return self.select_single("TimerPage").wait_select_single("DialerHand", objectName="minuteHand")

    def get_preset_label_text(self):
        """Returns the label with the preset text"""
        preset = self.select_many("Label", objectName="presetTextLabel")
        return preset[0]  # last preset, the one created by autopilot

    def get_preset_label_timer(self):
        """Returns the label with the preset timer"""
        preset = self.select_many("Label", objectName="presetTimerLabel")
        return preset[0]  # last preset, the one created by autopilot

    def get_label_timer(self):
        """Return the label with the timer countdown"""
        return self.select_single("TimerPage").wait_select_single("Label", objectName="label")

    def get_first_preset_list_item(self):
        """Returns the first preset list item in the timer page."""
        return self.wait_select_single("Standard", objectName="preset0")

    def get_num_of_presets(self):
        """Returns the number of presets in the timer page."""
        timer_page = self.select_single("TimerPage")
        return int(timer_page.wait_select_single("QQuickListView").count)

    # STOPWATCH Page Emulator Functions
    def get_stopwatch_label(self):
        """Returns the select for the stopwatch label"""
        return self.select_single("StopwatchPage").wait_select_single("Label", objectName="label")

    def get_stopwatch_button(self):
        """Returns the select for the stopwatch button"""
        return self.wait_select_single("StopwatchFace",
                                  objectName="stopwatchFace")

    def get_lap_button(self):
        """Returns the select for the lap button"""
        return self.wait_select_single("ImageButton", objectName="lapButton")

    def get_reset_button(self):
        """Returns the select for the reset button"""
        return self.wait_select_single("ImageButton", objectName="resetButton")

    def get_num_of_laps(self):
        """Returns the number of laps in the stopwatch page."""
        stopwatch_page = self.select_single("StopwatchPage")
        return int(stopwatch_page.wait_select_single("QQuickListView").count)

    def get_first_laps_list_item(self):
        """Returns the first lap list item in the stopwatch page."""
        return self.wait_select_single("Standard", objectName="laps0")

    # ALARM Page Emulator Functions
    def get_addalarm_page(self):
        """Returns the add alarm page object"""
        return self.select_single("Page", objectName="addAlarmPage")

    def get_alarm_page(self):
        """Returns the alarm page object"""
        return self.select_single("AlarmPage")

    def get_alarm_name_label(self):
        """Returns the TextField where insert the name of the preset"""
        return self.select_single("LabelDots", objectName="addAlarmName")

    def get_label_alarm(self):
        """Return the label with the alarm"""
        return self.select_single("Label", objectName="labelAlarmSetup")

    def get_num_of_alarms(self):
        """Returns the number of alarms in the alarm page."""
        listview = self.select_single("QQuickListView",
                                      objectName="listSavedAlarm")
        #each alarm has 3 labels
        return len(listview.select_many("Label", visible=True)) / 3

        #TODO: QQuickListView.count does not work as expected
        #fix once this is working properly and use it

    #TODO: all of the get_first functions need to
    #be refactored where possible to get X item, not
    #just the first

    def get_first_alarm_list_item(self):
        """Returns the first alarm list item in the alarm page."""
        listview = self.select_single("QQuickListView",
                                      objectName="listSavedAlarm")
        return listview.select_many("Base")[0]

    def toggle_first_alarm(self, test_case):
        """Toggles the alarm on and off in the alarm page."""
        alarm = self.select_single("CheckBox",
                                   objectName="listAlarmStatus")
        self.pointing_device.click_object(alarm)

        test_case.assertThat(lambda: self.get_alarm_page().select_single(
                     "QQuickFlickable").moving,
                     Eventually(Equals(False)))

    def get_first_alarm_time(self):
        """Gets the first alarm time in the alarm page."""
        alarm = self.get_first_alarm_list_item()
        return alarm.select_single("Label",
                                   objectName="listAlarmTime").text

    def get_first_alarm_subtitle(self):
        """Gets the first alarm subtitle in the alarm page."""
        alarm = self.get_first_alarm_list_item()
        return alarm.select_single("Label",
                                   objectName="listAlarmSubtitle").text

    def get_first_alarm_label(self):
        """Gets the first alarm label in the alarm page."""
        alarm = self.get_first_alarm_list_item()
        return alarm.select_single("Label",
                                   objectName="listAlarmLabel").text

    def get_first_alarm_status(self):
        """Gets the first alarm checkbox status in the alarm page."""
        alarm = self.get_first_alarm_list_item()
        return alarm.select_single("CheckBox",
                                   objectName="listAlarmStatus").checked

    def get_alarm_minute_hand(self):
        """Returns the hour hand of clock in alarm tab"""
        return self.select_single("AlarmPage").select_single("DialerHand",
                                  objectName="minuteHand")

    def get_alarm_hour_hand(self):
        """Returns the hour hand of clock in alarm tab"""
        return self.select_single("AlarmPage").select_single("DialerHand",
                                  objectName="hourHand")

    def set_alarm_week(self, day, test_case):
        #TODO: add support for setting days of week
        dayselectDialog = self.select_single("Dialog",
                                             objectName="alarmDaysDialog")

        return dayselectDialog

    def _retry_selector_assert(self, clickobj, selector, status, test_case):
        test_case.assertThat(lambda: self.get_alarm_page().select_single(
                             "QQuickFlickable").moving,
                             Eventually(Equals(False)))
        timeout = 0
        test_case.assertThat(lambda: selector.select_single(
            "OptionSelectorDelegate", selected=True),
            Eventually(Not(Is(None))))

        option = selector.select_single("OptionSelectorDelegate",
                                        selected=True).state
        while option != status and timeout < 10:
            self.pointing_device.click_object(clickobj)
            option = selector.select_single("OptionSelectorDelegate",
                                            selected=True).state
            sleep(1)
            timeout += 1

        test_case.assertThat(lambda: self.get_alarm_page().select_single(
                             "QQuickFlickable").moving,
                             Eventually(Equals(False)))

    def set_alarm_day(self, day, test_case):
        select = self.select_single("OptionSelector",
                                    objectName="OccursSelector")
        occur = select.select_single("StyledItem", objectName="listContainer")
        #optionslist = occur.select_single("OptionSelectorDelegate",
        #                                   selected=True)
        occurlist = occur.select_many("Label", visible=True)

        self.pointing_device.click_object(select)
        self._retry_selector_assert(select, occur,
                                    "dividerExpanded", test_case)

        self.drag_page_up("AlarmPage", test_case)
        self.pointing_device.click_object(occurlist[day])
        self._retry_selector_assert(select, occur,
                                    "dividerCollapsed", test_case)

        #check index
        #TODO: clock doesn't put things in in the proper order atm
        #test_case.assertThat(lambda: self.select_single("OptionSelector",
        #                     objectName="OccursSelector").selectedIndex+1,
        #                     Eventually(Equals(day)))

    def set_alarm_frequency(self, occurence, test_case):
        select = self.select_single("OptionSelector",
                                    objectName="RepeatSelector")
        repeat = select.select_single("StyledItem",
                                      objectName="listContainer")
        #optionslist = occur.select_single("OptionSelectorDelegate",
        #                                   selected=True)
        repeatlist = repeat.select_many("Label", visible=True)

        self.pointing_device.click_object(repeat)
        self._retry_selector_assert(select, repeat,
                                    "dividerExpanded", test_case)

        self.drag_page_up("AlarmPage", test_case)
        self.pointing_device.click_object(repeatlist[occurence])
        self._retry_selector_assert(select, repeat,
                                    "dividerCollapsed", test_case)

        #check index
        test_case.assertThat(lambda: self.select_single(
            "OptionSelector", objectName="RepeatSelector").selectedIndex+1,
            Eventually(Equals(occurence)))

