# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2013 Canonical Ltd
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Authored by: Nekhelesh Ramananthan <krnekhelesh@gmail.com>

"""Tests for the Clock App - Stopwatch"""

from __future__ import absolute_import

from testtools.matchers import Equals, NotEquals, Is, Not
from autopilot.matchers import Eventually

from ubuntu_clock_app.tests import ClockAppTestCase

class TestStopwatch(ClockAppTestCase):
    """Tests the stopwatch page features"""

    """ This is needed to wait for the application to start.
        In the testfarm, the application may take some time to show up."""
    def setUp(self):
        super(TestStopwatch, self).setUp()
        self.assertThat(
            self.main_view.visible, Eventually(Equals(True)))

        # Move to stopwatch tab
        self.main_view.switch_to_tab("StopwatchTab")

    def tearDown(self):
        super(TestStopwatch, self).tearDown()

    def create_lap(self):
        """Function to create a stopwatch lap"""
        lap_button = self.main_view.get_lap_button()
        self.pointing_device.click_object(lap_button)

    def start_stop_stopwatch(self):
        """Function to start/stop the stopwatch"""
        start_stop_button = self.main_view.get_stopwatch_button()
        self.pointing_device.click_object(start_stop_button)        

    def test_start_stop_reset_stopwatch(self):
        """Test to check the proper functioning of the start/stop/reset of stopwatch"""

        stopwatch_label = self.main_view.get_stopwatch_label()
        reset_button = self.main_view.get_reset_button()

        # Start the stopwatch and check if the stopwatch starts ticking
        stopwatch_label_before = self.main_view.get_stopwatch_label().text
        self.start_stop_stopwatch()
        self.assertThat(stopwatch_label.text, Eventually(NotEquals("00:00.0")))

        # Stop the stopwatch and check it has indeed stopped
        self.start_stop_stopwatch()
        self.assertThat(stopwatch_label_before, NotEquals(self.main_view.get_stopwatch_label().text))
        stopwatch_label_after = lambda: self.main_view.get_stopwatch_label().text
        self.assertThat(stopwatch_label_after, Eventually(Equals(self.main_view.get_stopwatch_label().text)))

        # Press reset and check if the stopwatch has been resetted
        self.pointing_device.click_object(reset_button)
        self.assertThat(stopwatch_label.text, Eventually(Equals("00:00.0")))

    def test_create_lap(self):
        """Test to check creating a new stopwatch lap"""

        # Start the stopwatch
        stopwatch_label = self.main_view.get_stopwatch_label().text
        self.start_stop_stopwatch()

        # Ensure that the stopwatch actually ticks
        self.assertThat(stopwatch_label, Eventually(NotEquals("00:00.0")))

        # Create a stopwatch lap by pressing the lap button
        num_of_laps_old = self.main_view.get_num_of_laps()
        self.create_lap()

        # Confirm that the lap is created
        self.assertThat(
            self.main_view.get_num_of_laps,
            Eventually(Equals(num_of_laps_old + 1)))

    def test_delete_lap(self):
        """Test to check deleting a stopwatch lap"""

        # Start the stopwatch
        stopwatch_label = self.main_view.get_stopwatch_label().text
        self.start_stop_stopwatch()

        # Ensure that the stopwatch actually ticks
        self.assertThat(stopwatch_label, Eventually(NotEquals("00:00.0")))

        # Create a stopwatch lap
        self.create_lap()

        num_of_laps_old = self.main_view.get_num_of_laps()
        self.main_view.drag_page_up("StopwatchPage", self)

        # Delete stopwatch lap
        first_lap = self.main_view.get_first_laps_list_item()
        first_lap.swipe_to_delete()
        first_lap.confirm_removal()

        # Check that the stopwatch lap has been deleted
        self.assertThat(
            self.main_view.get_num_of_laps,
            Eventually(Equals(num_of_laps_old - 1)))
