/*
 * Copyright (C) 2013, 2014 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Alessandro Pozzi <signor.hyde@gmail.com>
 *              Nekhelesh Ramananthan <krnekhelesh@gmail.com>
 */

import QtQuick 2.0
import U1db 1.0 as U1db
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import Ubuntu.Components.Themes.Ambiance 0.1
import "../common/ClockUtils.js" as Utils
import "../common"

Page {
    id: timerPage

    property alias minutes: analogTimer.minutes;
    property alias seconds: analogTimer.seconds;
    property alias totalTime: analogTimer.totalTime;
    property alias timerOn: analogTimer.timerOn;
    property real tempPosition
    property bool setFocus: false

    // Default Timer Presets as defined by the design team
    U1db.Document {
        database: db
        docId: "defaultTimerOne"
        create: true
        defaults: { "timer": { "name": i18n.tr("Soft-boiled egg"), "duration": 180 }}
    }

    U1db.Document {
        database: db
        docId: "defaultTimerTwo"
        create: true
        defaults: { "timer": { "name": i18n.tr("Aeropress coffee"), "duration": 165 }}
    }

    U1db.Document {
        database: db
        docId: "defaultTimerThree"
        create: true
        defaults: { "timer": { "name": i18n.tr("Al dente"), "duration": 600 }}
    }

    U1db.Index {
        database: db
        id: by_timer
        expression: ["timer.name", "timer.duration"]
    }

    U1db.Query {
        id: timers
        index: by_timer
        query: ["*", "*"]
    }

    // Function to add a timer preset
    function add_preset() {
        reset()
        timerPage.state = "addPreset"
        if(timerPage.setFocus)
            namePreset.forceActiveFocus()
    }

    // Function to cancel a timer preset
    function cancel_preset() {
        if(timerPage.state === "addPreset") {
            reset()
            timerPage.state = "";
            timerPage.setFocus = false
            toolbarTimer.opened = false;
        }
    }

    // Function to save timer presets to u1db
    function savePreset(label, seconds, index, docId) {
        if(index === -1) {
            Utils.log("Saving a new preset")
            db.putDoc({ "timer": { "name": label, "duration": seconds }})
        }
        else {
            Utils.log("Updating an existing preset")
            db.putDoc({ "timer": { "name": label, "duration": seconds }}, listPreset.listDocId)
        }
    }

    Connections {
        target: listPreset
        // On deleting a timer preset, if there are no presets drag timer page up.
        onListCountChanged: listPreset.listCount === 0 ? timerAnimationContainer.contentY = tempPosition : undefined
    }

    actions:
        Action {
        id: addTimerAction
        iconSource: Qt.resolvedUrl("../images/add_icon.png")
        text: i18n.tr("Add Timer")
        description: i18n.tr("Add a new timer")
        // TRANSLATORS: this is a list of semicolon-separated keywords to be used to activate this action (Add Timer) in the HUD
        keywords: i18n.tr("Add;Timer;Eggtimer;Preset;Ticker")
        onTriggered: add_preset()
    }

    states: [
        State {
            name: "selectPreset"
            when: listPreset.listIndex != -1
            PropertyChanges { target: namePreset; visible: true }
            PropertyChanges { target: editPresetToolbarButton; visible: true }
            PropertyChanges { target: addPresetToolbarButton; visible: false }
        },
        State {
            name: "addPreset"
            PropertyChanges { target: analogTimer; enableMouseArea: false }
            PropertyChanges { target: listPreset; visible: false }
            PropertyChanges { target: namePreset; visible: true }
            // TODO: Fix autopilot test first before enabling this change in another MP
            //PropertyChanges { target: namePreset; text: "Timer #" + (listPreset.listCount + 1) }
            PropertyChanges { target: addPresetToolbarButton; visible: false }
            PropertyChanges { target: editPresetToolbarButton; visible: true }
            PropertyChanges { target: cancelPresetToolbarButton; visible: true }
            PropertyChanges { target: toolbarTimer; locked: true }
            PropertyChanges { target: toolbarTimer; opened: true }
        }
    ]

    function onTimerUpdate() {
        analogTimer.onTimerUpdate();
    }

    // Function to print timer label (format mm:ss and hh:mm:ss when hh > 0) at top
    function setTimerLabel(minutes, seconds) {
        if (minutes > 59) { // Do not show "60" value for minutes
            minutes = 0
        }
        if (seconds > 59) { // Do not show "60" value for seconds
            seconds = 0
        }
        analogTimer.minutes = Math.round(minutes) // This makes minute hand stick to digits
        analogTimer.seconds = Math.round(seconds) // This makes second hand stick to digits
        return Utils.zeroleft(minutes, 1) + ":" + Utils.zeroleft(seconds, 2);
    }

    function reset() {
        analogTimer.reset();
        listPreset.listIndex = -1;
        namePreset.text = "";
    }

    Component.onCompleted: {
        Utils.log("TimerPage loaded");
        tempPosition = timerAnimationContainer.contentY
    }

    Flickable {
        id: timerAnimationContainer
        objectName: "animationContainer"

        clip: true;
        anchors.fill: parent
        interactive: {      // Flickable interaction disabled if there are no presets and not in add timer page.
            if (listPreset.listCount == 0) {
                if (timerPage.state == "addPreset")
                    return true
                else
                    return false
            }
            else
                return true
        }
        contentWidth: parent.width
        contentHeight: analogTimer.height + analogTimer.anchors.topMargin + listPreset.height + listPreset.anchors.topMargin + units.gu(2)

        // Main element which draws the timer face and manages time.
        TimerFace {
            id: analogTimer

            anchors {
                top: parent.top
                topMargin: units.gu(10)
                horizontalCenter: parent.horizontalCenter
            }

            innerLabel.text: setTimerLabel(minutes, seconds);

            Timer {
                id: seconds_timer
                interval: 250
                repeat: true
                onTriggered: analogTimer.innerLabel.text = setTimerLabel(minutes, seconds);
            }

            inProgressFlag: (timerOn === true || analogTimer.state === "DONE") ? true : false;
            onMinutesChanged: innerLabel.text = setTimerLabel(minutes, seconds);
            onSecondsChanged: if (timerOn == true) seconds_timer.restart()
                              else innerLabel.text = setTimerLabel(minutes, seconds);
            states:[
                State { name: "STOP" },
                State { name: "PAUSE"},
                State {
                    name: "DONE"
                    when: (totalTime == -1 && timerOn == false)
                    PropertyChanges { target: analogTimer; timerHintImage: Qt.resolvedUrl("../images/timer_green.png") }
                }
            ]

            onClicked: {
                switch (analogTimer.state) {

                case "":
                    if ((seconds != 0 || minutes != 0) || timerOn) {
                        totalTime = (Math.round(minutes) * 60) + seconds;
                        analogTimer.startTime = new Date();
                        timerOn = true;
                        analogTimer.state = "STOP";
                    }
                    break;

                case "STOP":
                    timerOn = false;
                    analogTimer.pauseTime = analogTimer.totalTime - analogTimer.remTime + 1; // +1 is necessary to start immediately the timer when finish a pause
                    analogTimer.state = "PAUSE";
                    break;

                case "DONE":
                    reset();
                    analogTimer.state = "";
                    break;

                case "PAUSE":
                    analogTimer.startTime = new Date();
                    timerOn = true;
                    analogTimer.state = "STOP";
                    break;
                }
            }
        }

        // Element to set the name of a saved preset.
        LabelDots {
            id: namePreset
            objectName: "namePreset"

            visible: false
            width: length < (analogTimer.width - units.gu(4))/units.gu(1.5)  ? length * units.gu(1.5) + units.gu(4) : analogTimer.width;
            placeholderText: i18n.tr("Preset Name")
            anchors { horizontalCenter: analogTimer.horizontalCenter; top: analogTimer.bottom; topMargin: units.gu(2) }
            onClickedOutside: analogTimer.forceActiveFocus()
        }

        // Reset button to reset timer
        ImageButton {
            id: resetButtonTimer
            objectName: "resetButtonTimer"

            anchors { top: analogTimer.bottom; topMargin: units.gu(7); horizontalCenter: analogTimer.horizontalCenter; leftMargin: units.gu(4) }
            buttonImage: Qt.resolvedUrl("../images/reset_icon.svg");
            buttonLabel: i18n.tr("Reset");
            visible: timerOn || (minutes != 0 || seconds != 0)? true : false;
            enabled: true

            onClicked: {
                reset();
                if (timerPage.state != "addPreset")
                    timerAnimationContainer.contentY = tempPosition
            }
        }

        // Column element to hold the saved presets
        PresetList {
            id: listPreset
            model: timers
            visible: listCount !== 0
            anchors { left: parent.left; right: parent.right; top: analogTimer.bottom; topMargin: listPreset.dynamicTopSpacing }
            dynamicTopSpacing: timerPage.height - timerPage.header.height - analogTimer.height - analogTimer.anchors.topMargin - headerHeight - dividerHeight;
        }
    }

    // Various toolbar actions such as add preset, edit and back.
    tools: ToolbarItems {
        id: toolbarTimer

        opened: false
        locked: false

        ToolbarButton {
            id: editPresetToolbarButton
            objectName: "savePresetButton"
            visible: false
            iconSource: Qt.resolvedUrl("../images/select_icon.png")
            text: i18n.tr("Save")
            onTriggered: {
                savePreset(namePreset.text, ((minutes * 60) + seconds), listPreset.listIndex, listPreset.listDocId)
                namePreset.focus = false
                if (listPreset.listIndex == -1) {
                    reset()
                    timerPage.state = "";
                    timerAnimationContainer.contentY = tempPosition
                }
                timerPage.setFocus = false
                toolbarTimer.opened = false;
            }
        }

        ToolbarButton {
            id: addPresetToolbarButton
            objectName: "addPresetButton"
            action: addTimerAction
        }

        ToolbarButton {
            id: cancelPresetToolbarButton
            objectName: "cancelPresetButton"
            visible: false;
            iconSource: Qt.resolvedUrl("../images/cancel_icon.svg")
            text: i18n.tr("Cancel")
            onTriggered: cancel_preset()
        }
    }
}
