/*
 * Copyright (C) 2013, 2014 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Juha Ristolainen <juha.ristolainen@codemancers.fi>
 *              Nekhelesh Ramananthan <krnekhelesh@gmail.com>
 *              Nick Leppänen Larsson <frals@frals.se>
 *              Marco Biscaro <marcobiscaro2112@gmail.com>
 *              Alessandro Pozzi <signor.hyde@gmail.com>
 */

import QtQuick 2.0
import U1db 1.0 as U1db
import Ubuntu.Components 0.1
import "common/ClockUtils.js" as Utils
import "common/KeyboardShortcuts.js" as Keyboard
import "clock"
import "alarm"
import "stopwatch"
import "timer"

MainView {
    id: root
    objectName: "clock"

    applicationName: "com.ubuntu.clock"

    width: minimumWidth
    height: minimumHeight

    // StateSaver properties
    StateSaver.properties: "width, height"

    headerColor: "#57365E"
    backgroundColor: "#A55263"
    footerColor: "#D75669"

    focus: true
    Keys.onReturnPressed: Keyboard.enterShortcut(event, rootTabs.selectedTabIndex, stopwatchPage)
    Keys.onEnterPressed: Keyboard.enterShortcut(event, rootTabs.selectedTabIndex, stopwatchPage)
    Keys.onSpacePressed: Keyboard.spaceShortcut(event, rootTabs.selectedTabIndex, stopwatchPage)
    Keys.onEscapePressed: Keyboard.escapeShortcut(event, rootTabs.selectedTabIndex, pagestack, timerPage, stopwatchPage)
    Keys.onPressed: rootTabs.selectedTabIndex = Keyboard.generalShortcuts(event, rootTabs.count, rootTabs.selectedTabIndex, pagestack, timerPage, stopwatchPage)

    // Property to store the default application dimensions
    readonly property real minimumWidth: units.gu(50)
    readonly property real minimumHeight: units.gu(75)

    // Property to store the state of an application (active or suspended)
    property bool applicationState: Qt.application.active

    // Property to store the current time
    property var now

    // Initialization after page has loaded
    Component.onCompleted: {
        // Any init here
        Utils.log("MainView loaded");
        // When root component is completed the StateSaver restore it to previous dimensions.
        // If it is too small, set height and width to a proper size.
        if((root.width < minimumWidth) || (root.height < minimumHeight)){
            root.width = minimumWidth
            root.height = minimumHeight
        }
    }

    actions: [
        Action {
            id: appSettingsAction
            text: i18n.tr("Settings")
            // TRANSLATORS: this is a list of semicolon-separated keywords to be used to activate this action (Settings) in the HUD
            keywords: i18n.tr("Settings;Setting;Configuration;")
            description: i18n.tr("Application Settings")
            iconSource: Qt.resolvedUrl("images/settings.svg")
            onTriggered: pagestack.push(Qt.resolvedUrl("common/SettingsPage.qml"))
        }
    ]

    // Timer that runs our app
    Timer {
        id: updateTimer
        objectName: "updateTimer"

        interval: 1000
        repeat: true
        running: true

        onTriggered: {
            now = new Date();
            // FIXME: only send callback to pages that need it
            /* TODO: this timer should probably be changed to fire
              every minute (max) when the app is in the background,
              or just be activated when the app is visible in the
              switcher
              */
            clockPage.onTimerUpdate(now);
            alarmPage.onTimerUpdate(now);
            timerPage.onTimerUpdate(now);

            /* stopwatchPage.onTimerUpdate(now) is not required since
               there is an inbuilt stopwatch timer which runs at a
               higher frequency.
             */
        }
    }

    // Database to store app data locally
    U1db.Database {
        id: db
        path: "settings"
    }

    // Document to store app settings data
    U1db.Document {
        id: appSetting
        database: db
        docId: "appSettings"
        create: true
        defaults: { "timeFormat": "12-hour", "style": "analogue" }
    }

    ////////////////////////////////////////////////////////////////////////////////
    // UI
    PageStack {
        id: pagestack

        Component.onCompleted: push(rootTabs);

        // Applications pages in tabs (Clock, Alarm, Timer, Stopwatch)
        Tabs {
            id: rootTabs
            objectName: "rootTabs"
            anchors.fill: parent

            Tab {
                objectName: "ClockTab"

                title: i18n.tr("Clock")

                // Tab content begins here
                page: ClockPage {
                    id: clockPage
                }
            }

            // Disabled Alarm Tab until the feature is ready for use.
            Tab {
                objectName: "AlarmTab"

                title: i18n.tr("Alarm")

                page: AlarmPage {
                    id: alarmPage

                    AlarmModel {
                        id: alarmModel
                        Component.onCompleted: Utils.log("Alarm Database loaded")
                        onModelReset: alarmPage.get_next_active_alarm()
                    }
                }
            }

            Tab {
                objectName: "TimerTab"

                title: i18n.tr("Timer")

                page: TimerPage {
                    id: timerPage
                }
            }

            Tab {
                objectName: "StopwatchTab"

                title: i18n.tr("Stopwatch")

                page: StopwatchPage {
                    id: stopwatchPage
                }
            }
        }
    }
}
